import { ComponentPropsWithoutRef } from 'react';
import { PolymorphicAs } from '../../Polymorphic';
/**
 * Returns a loosely typed prop object,
 * with a defined `as` value,
 * a potentially defined `href` prop,
 * and the rest of the provided props,
 * typed as a union of attributes on all PolymorphicAs value
 *
 * For client-side components, prefer using `useInferredPolymorphic`,
 * which also returns a typed `ref`
 */
export declare function useInferredPolymorphicProps<T extends PolymorphicAs, R extends {
    [key: string]: any;
}, D extends PolymorphicAs>(asProp?: T, rest?: R, defaultAs?: D): {
    /** The element or component to render as */
    as: PolymorphicAs;
    /** The URL that the hyperlink points to */
    href?: string;
} & ComponentPropsWithoutRef<PolymorphicAs>;
//# sourceMappingURL=useInferredPolymorphicProps.d.ts.map